--[[
    FuBar_MagicMarker - MagicMarker FuBar plugin
--]]

MMFu = AceLibrary("AceAddon-2.0"):new("FuBarPlugin-2.0", "AceDB-2.0")
local L = LibStub("AceLocale-3.0"):GetLocale("MagicMarker", false)
local C = LibStub("AceConfigDialog-3.0")
local T = AceLibrary("Tablet-2.0")

local MMFu = MMFu
local MM = MagicMarker
local MMDB = MagicMarkerDB

MMFu:RegisterDB("MMFuDB")
MMFu:RegisterDefaults('profile', { hideTitle = false })

MMFu.hasIcon = true
MMFu.canHideText = true
MMFu.canHideIcon = true
MMFu.blizzardTooltip = false
MMFu.cannotDetachTooltip = true


local mod = mod
local tinsert = tinsert
local sort = table.sort

local db

local options = {
   type = "group",
   args = {
      config = { 
	 type = "toggle",
	 name =  L["Toggle config dialog"],
	 desc = L["Toggle the Magic Marker configuration dialog."],
	 set = "ToggleConfigDialog",
	 get = function() return C.OpenFrames["Magic Marker"] ~= nil end,
	 handler = MM,
	 order = 1
      }, 
      toggle = {
	 type = "toggle", 
	 name = L["Toggle event handling"],
	 desc = L["Enable or disable the event handling, i.e whether or not Magic Marker will insert mobs into the mob database, mark mobs etc."],
	 set = "ToggleMagicMarker",
	 get = function() return MM.addonEnabled end,
	 handler = MM,
	 order = 2
      },
      assignments = {
	 type = "execute",
	 name = L["Report the raid icon assignments to raid/party chat"]..".",
	 desc = L["Report the raid icon assignments to raid/party chat"]..".",
	 func = "ReportRaidMarks",
	 handler = MM
      },
      reset = {
	 type = "execute",
	 name = L["Reset raid icon cache"],
	 desc = L["Reset the raid icon cache. Note that this honors the Magic Marker options such as preserve raid marks."],
	 func = "ResetMarkData",
	 handler = MM,
      },
      commsettings = {
	 type = "group",
	 name = L["Data Sharing"],
	 desc = L["Data Sharing"],
	 order = 1200,
	 args = {}
      },
      settings = { 
	 type = "group",
	 name = L["General Options"],
	 desc = L["General Options"],
	 order = 1100,
	 args = {}
      },
      hideTitle = {
	 type = "toggle",
	 name = L["Short FuBar Text"],
	 desc = L["Hide Magic Marker from the FuBar status text."],
	 set = function() db.hideTitle = not db.hideTitle MMFu:Update() end,
	 get = function() return db.hideTitle end,
      },

      cache = {
	 type = "group",
	 name = L["Raid mark layout caching"],
	 desc = L["RAIDMARKCACHEHELP"],
	 order = 1000,
	 handler = MM,
	 args = {
	    save = {
	       type = "execute",
	       name = L["Save party/raid mark layout"],
	       desc = L["Save the current raid mark layout."],
	       func = "CacheRaidMarks",
	    },
	    load = {
	       type = "execute",
	       name = L["Load party/raid mark layout"],
	       desc = L["Load the currently saved raid mark layout."],
	       func = "MarkRaidFromCache",
	    },
	 }
      },
      tmpl = {
	 type = "group",
	 name = L["Raid group target templates"],
	 desc  = L["RAIDTMPLHELP"],
	 handler = MM,
	 order = 900
      },
   }
}
MMFu.OnMenuRequest = options

function MMFu:OnProfileEnable()
   db = self.db.profile 
end

local function capitalize(str)
   return strupper(str.sub(str,1,1))..str.sub(str, 2)
end



function MMFu:OnInitialize()
   db = self.db.profile
   if not options.args.tmpl.args then
      args = {}
      for cmd,data in pairs(MM.MarkTemplates) do
	 if cmd ~= "arch" then
	    args[cmd] = {
	       type = "execute",
	       name = string.format("%s: %s", capitalize(cmd), data.desc),
	       desc = data.desc,
	       func = function() MM:MarkRaidFromTemplate(cmd) end,
	    }
	 end
      end
      options.args.tmpl.args = args
   end
   local mmopts = MM:GetOptions()
   local tmpdata
   for option, data in pairs(mmopts.args.options.args.settings.args) do
      if data.type ~= "select" and data.type ~= "header" then
	 tmpdata = {}
	 -- Need to copy the table since we're modifying it
	 for key,val in pairs(data) do
	    tmpdata[key] = val
	 end
	 tmpdata.get = function() return MM:GetProfileParam({option}) end
	 tmpdata.set = function(val) MM:SetProfileParam({option},val) MM:NotifyChange() end
	 options.args.settings.args[option] = tmpdata
      end
   end

   for option, data in pairs(mmopts.args.options.args.commsettings.args) do
      if data.type ~= "select" and data.type ~= "header" then
	 tmpdata = {}
	 -- Need to copy the table since we're modifying it
	 for key,val in pairs(data) do
	    tmpdata[key] = val
	 end
	 if not tmpdata.desc then tmpdata.desc = tmpdata.name end
	 local var = {option}
	 tmpdata.get = function() return MM:GetProfileParam(var) end
	 tmpdata.set = function(val) MM:SetProfileParam(var,val) MM:NotifyChange() end
	 options.args.commsettings.args[option] = tmpdata
      end
   end
end

local function white(str)
   return string.format("|cffffffff%s|r", str)
end

local function yellow(str)
   return string.format("|cffffff00%s|r", str)
end

function MMFu:OnTextUpdate()
   if db.hideTitle then
      self:SetText(white((MM.addonEnabled and L["Enabled"]) or L["Disabled"]))
   else
      self:SetText(string.format("%s: |cffffffff%s|r", L["Magic Marker"],
				 (MM.addonEnabled and L["Enabled"]) or L["Disabled"]))
   end
end

function MMFu:OnClick()
   if IsAltKeyDown() then
      MM:ResetMarkData(IsShiftKeyDown())
   elseif IsShiftKeyDown() then
      MM:ToggleMagicMarker()
   else
      MM:ToggleConfigDialog()
   end   
end

function MMFu:OnMouseUp()
   if arg1 == "MiddleButton" then
      MM:ReportRaidMarks()
   end
end

function MMFu:OnTooltipUpdate()
   local zone = GetRealZoneText()
   local supercat = T:AddCategory('columns', 2)
   
   supercat:AddLine( 'text', L["Status"],
	       'text2', white((MM.addonEnabled and L["Enabled"]) or L["Disabled"]))
   supercat:AddLine( 'text', L["Zone"],'text2', white(zone))

   local zoneID, _, heroic = MM:GetZoneName(zone)
   local inInstance, type = IsInInstance()
   if inInstance then
      if type == "raid" then
	 supercat:AddLine( 'text', L["Difficulty"], 'text2', L["Raid"])
      elseif type == 'pvp' or type == 'arena' then
	 supercat:AddLine( 'text', L["Difficulty"], 'text2', "PvP")
      else
	 supercat:AddLine( 'text', L["Difficulty"],
			  'text2', white((heroic and L["Heroic"]) or L["Normal"]))
      end
   end
   local cat
   if MMDB.mobdata[zoneID] then
      cat = T:AddCategory(nil):AddLine('text', MM:GetZoneInfo(MMDB.mobdata[zoneID]))
   end
   
   local sortData = {}
   local hasData
   local cacheData = MM:GetMarkData()
   for id, data in pairs(cacheData) do
      if data.value then
	 tinsert(sortData, data.value * 10 + id)
	 hasData = true
      end
   end
   if hasData then
      cat = T:AddCategory('text', 'Raid Mark Cache',  'columns', 3)
      sort(sortData, function(a,b) return a > b end)   
      cat:AddLine('text', yellow(L["Unit Name"]),
		  'text2', yellow(L["Mark Type"]), 
		  'text3', yellow(L["Score"]),
		  'size', 10, 'size2', 10, 'size3', 10, 
		  'justify2', 'CENTER',
		  'justify3', 'RIGHT')
      local data
      for _, id in pairs(sortData) do
	 id = mod(id, 10)
	 data = cacheData[id]
	 if data.uid then
	    unitData = MM:GetUnitHash(data.uid)
	    cat:AddLine('text', unitData and unitData.name or data.uid,
			'text2', MM:GetCCName(data.ccid, data.value),
			'text3', tostring(data.value),
			'size', 10, 'size2', 10, 'size3', 10, 
			'justify2', 'CENTER',
			'justify3', 'RIGHT',
			'hasCheck', true,
			'checkIcon', MM:GetIconTexture(id),
			'checked', true
		     )
	 end
      end
   end
   
   T:SetHint(L["TOOLTIP_HINT"])
end
